% The MIT License (MIT)
% Copyright (c) 2016 Elmar Rueckert, Daniel Tanneberg, David Kappel
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE. 

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: rueckert.elmar@gmail.com, tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at
% *
% **********************************************************

%add subfolders
addpath([pwd filesep 'cbrewer']);                   %for plotting advanced colormaps
addpath(genpath([pwd filesep 'Programs']));         %the demo implementations
addpath([pwd filesep 'SpikingNetworkFunctions']);   %shared functions for generating, 
                                                    %using and training the recurrent neural network
clear all;
close all;
clc;

%% Program One
disp('% **********************************************************');
disp('% * ');
disp('% * 1) Neural Network Sampling with Soft Winner-Take-All Circuits');
disp('% * ');
% * Description: Demonstrates how the recurrent dynamics can be used to 
%   generate spike trains that implement random walks. The behavior can
%   be controlled via the temperature.
% **********************************************************
fprintf('Press any key to start...\n');
pause

run('RecurrentSamplingDemo.m');

if input(sprintf('The demo terminated. Do you want to continue? [y/n]'),'s') == 'n'
    return;
end

%% Program Two
disp('% **********************************************************');
disp('% * ');
disp('% * 2) Imitation learning using a spiking contrastive divergence implementation');
disp('% * ');
% * Description: This demo shows how a spiking update rule of contrastive
% divergence can be used for supervised model learning. This is
% demonstrated in a transition model learning task using Cartesian
% End-Effector trajectories from a KUKA LWR robot arm. The continuous
% trajectories were transformed into spike trains using a population code
% with Gaussian basis functions and poisson point neurons. 
% **********************************************************
fprintf('Press any key to start...\n');
pause

run('ImitationLearningDemo.m');

if input(sprintf('The demo terminated. Do you want to continue? [y/n]'),'s') == 'n'
    return; 
end

%% Program Three
disp('% **********************************************************');
disp('% * ');
disp('% * 3) Reinforcement Learning using one input neuron per time step');
disp('% * ');
% * Description: This demo shows how a simple recurrent neural network 
%   can be used to solve the finite horizon planning problem illustrated
%   in Figure 1C in the paper. 
% **********************************************************
fprintf('Press any key to start...\n');
pause

run('ReinforcementLearningTVInputDemo.m');

if input(sprintf('The demo terminated. Do you want to continue? [y/n]'),'s') == 'n'
    return;
end

%% Program Four
disp('% **********************************************************');
disp('% * ');
disp('% * 4) Reinforcement Learning using one input neuron per maze location');
disp('% * ');
% * Description: This demo shows how a simple recurrent neural network 
%   can be used to solve the infinite horizon planning problem illustrated
%   in Figure 2 in the paper. Illustrated are the value function updates with
%   the online and the offline learning rules. 
% **********************************************************
fprintf('Press any key to start...\n');
pause

run('ReinforcementLearningTINVInputDemo.m');

if input(sprintf('The demo terminated. Do you want to continue? [y/n]'),'s') == 'n'
    return;
end

%% Program Five
disp('% **********************************************************');
disp('% * ');
disp('% * 5) Robot Reaching Experiment with learned models');
disp('% * ');
% * Description: This Demo shows how motion planning in continues states 
%   can be implemented in the recurrent net. The transition model is 
%   assumed to be Gaussian. The inferred path from a know initial 
%   location to a desired goal location is applied to a planar two-link
%   robot arm simulation.  
% **********************************************************
fprintf('Press any key to start...\n');
pause

run('RealRobotReachingDemo.m');

