% The MIT License (MIT)
% Copyright (c) 2016 Elmar Rueckert, Daniel Tanneberg, David Kappel
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE. 

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: rueckert.elmar@gmail.com, tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at
% *
% **********************************************************
function [transmat stateXContVal stateYContVal, K, X, Y, obstacleStates, obstacleStateIds] = GetMDPModelMaze(Kx, Ky, sigma, flgObstaclesAreKnown)

if nargin < 4
    flgObstaclesAreKnown = 0;
end
% Two DIMENSIONAL
%infinite horizon mdp problems 
Dimension = 2;
K = Kx*Ky;
%specify the transition model
Q = sigma*eye(Dimension);

X = zeros(K,1);
Y = zeros(K,1);
transmat = zeros(K,K);
stateXContVal = linspace(1, Kx, Kx);
stateYContVal = linspace(1, Ky, Ky);

id = 1;
for xId=1:Kx
   for yId = 1:Ky
      X(id,1) = stateXContVal(xId);
      Y(id,1) = stateYContVal(yId);

      id = id + 1; 
   end
end


 %check for wall conditions
obstacleXIds = [1:Kx, ...           %lower wall
                1:Kx,...            %upper wall
                ones(1,Ky-2), ...     %left wall
                Kx*ones(1,Ky-2), ...  %right wall
                8*ones(1,6), ...    %obstacle 1
                8*ones(1,4), ...    %obstacle 2
                6:10, ...           %obstacle 3
                ];
obstacleYIds = [ones(1,Kx),...        %lower wall
                Ky*ones(1,Kx),...     %upper wall
                2:Ky-1, ...         %left wall
                2:Ky-1, ...         %right wall
                9:14, ...           %obstacle 1
                4:7, ...            %obstacle 2
                3*ones(1,5), ...    %obstacle 3
                ];

obstacleStates = [obstacleXIds; obstacleYIds];
obstacleStateIds = zeros(size(obstacleStates,2),1);
for i=1:size(obstacleStates,2)
    XisZeroIds = find(abs(-obstacleStates(1,i) + X)<1e-6);
    YisZeroIds = find(abs(-obstacleStates(2,i) + Y)<1e-6);
    obstacleStateIds(i) = intersect(XisZeroIds, YisZeroIds);
end



dist = zeros(2,K);

for i=1:K
   prevState = [X(i,1); Y(i,1)];
   curState = [X(:,1)'; Y(:,1)'];
   
   if 1==0 %Gaussian cont. space transition model
        d = mvnpdf(curState',prevState',Q) + 1e-6; 
        transmat(i,:) = d';  
   else
        %Only allow left/right or up/down movements, where dist < sqrt(2)
        for xyId=1:2
            dist(xyId,:) = (prevState(xyId) - curState(xyId,:)).^2;
        end 
        dist = sqrt(sum(dist));
        valid = find(dist < sqrt(2));
        transmat(i,valid) = dist(valid); 
        %
   end
   %no self connections or agent can not rest at current state
   transmat(i,i) = 1e-6;
   
   if flgObstaclesAreKnown
       %keyboard
       transmat(i,intersect(obstacleStateIds',valid)) = 0;%0.1;%1e-6;
   end
end
transmat = mk_stochastic(transmat);

%not needed as we sample directly from ptrans*ptask
%transmat = log(transmat);



end