% The MIT License (MIT)
% Copyright (c) 2016 Elmar Rueckert, Daniel Tanneberg, David Kappel
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE. 

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: rueckert.elmar@gmail.com, tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at
% *
% **********************************************************

% % * Description: This demo shows how a simple recurrent neural network 
% %   can be used to solve the infinite horizon planning problem illustrated
% in Figure 2 in the paper. Illustrated are the value function updates with
% the online and the offline learning rules. 

function ReinforcementLearningTINVInputDemo()
global GLOBAL_OBJ_HANDLES

GLOBAL_OBJ_HANDLES.flgSimIsRunning = 0; 

%network parameters
net.N = 300;           % number of time steps
net.sigma = 0.5;%2;%0.2;
net.gamma = 0.98; 
net.eta = 1;
net.Kx = 15;    %number of neurons per x dimension
net.Ky = 20;    %number of neurons per y dimension
net.temperature = 1;
net.initState = [3; 2+9];
net.targetState = [13;2];
net.rewardTarget = 1;%10;
net.rewardObstacle = 0;
net.flgObstaclesAreKnown = 1;   %or learn the obstacles as well
[net.transmat, ~, ~, net.K, net.X, net.Y, net.obstacleStates, ~] = ...
    GetMDPModelMaze(net.Kx, net.Ky, net.sigma, net.flgObstaclesAreKnown);
net.wh= (net.Kx-1)/net.Kx;
net.XisZeroIds = find(abs(-net.initState(1) + net.X)<1e-6);
net.YisZeroIds = find(abs(-net.initState(2) + net.Y)<1e-6);
net.initStateId = intersect(net.XisZeroIds, net.YisZeroIds);
net.fnReward = @(stateIds_)fnRewardMazeToussaint06(stateIds_, ...
    net.initState, net.targetState, net.obstacleStates);
% net.fnReward = @(stateIds_)fnRewardMazeSingleState(stateIds_, ...
%     net.initState, net.targetState, net.obstacleStates, net.rewardTarget, net.rewardObstacle);
%network memory and results
net.rewards = [0];
net.succeeded = [0];
% net.maxRewardIds = zeros(1, num_trajSamples);
% net.flgHitObstacle = zeros(num_trajSamples,1);
% net.flgHitTarget = zeros(num_trajSamples,1);
% net.stateTrajectories = zeros(2,N,num_trajSamples);
% net.stateIdsVector = zeros(N,num_trajSamples);
% VtrainInit = -0.1* randn(net.K,1);
VtrainInit = 0* ones(net.K,1);%-1e-6* ones(net.K,1);
net.Vtrain = VtrainInit;
net.beta_est = exp(net.Vtrain); 
net.flgRunsFirstTime = 1;
net.maxReward = -9999;
net.bestStates = [];
net.bestMaxRewardId = 1;

%copy the net structure
GLOBAL_OBJ_HANDLES.netOnline = net;
GLOBAL_OBJ_HANDLES.netOffline = net;
GLOBAL_OBJ_HANDLES.netOffline.beta_estOffline = [];

%Create a figure to terminate the programm with the key 'q'
%*** Keystroke Event
% Define the key press callback fexitunction and open a figure to detect key strokes
GLOBAL_OBJ_HANDLES.flgSimIsRunning = 1;  
GLOBAL_OBJ_HANDLES.hFigKeyStrokes = figure; 
tmpPos = get(GLOBAL_OBJ_HANDLES.hFigKeyStrokes, 'position');
set(GLOBAL_OBJ_HANDLES.hFigKeyStrokes, 'position', [tmpPos(1), tmpPos(2), 1400, 400]);
set(GLOBAL_OBJ_HANDLES.hFigKeyStrokes, 'color', 'white');
set(GLOBAL_OBJ_HANDLES.hFigKeyStrokes,'KeyPressFcn',@KeyPressFcn);
%create the subplots

%ONLINE
GLOBAL_OBJ_HANDLES.netOnline.fhMaze = subplot(1,3,1);
set(GLOBAL_OBJ_HANDLES.netOnline.fhMaze,'Color','white');
hold on; plotMaze(net.Kx, net.Ky, net.wh); 
hold on; GLOBAL_OBJ_HANDLES.netOnline.PlotLikelihood = scatter(net.X, net.Y,0.1,...
    'c', 'filled', 'markeredgecolor', 'k', 'markerfacecolor','k');
hold on; scatter(  net.initState(1), net.initState(2), 50, 'x', 'linewidth', 3, 'markeredgecolor', 'r');
hold on; scatter( net.targetState(1), net.targetState(2), 50, 'x', 'linewidth', 3, 'markeredgecolor', 'k');
hold on; GLOBAL_OBJ_HANDLES.netOnline.PlotTrajectory = plot([0;0], [0;0], 'linewidth',2, 'color','b');
axis off;
axis([0,net.Kx+1,0,net.Ky+1]);
GLOBAL_OBJ_HANDLES.onlineTextBox = uicontrol('style','text');
set(GLOBAL_OBJ_HANDLES.onlineTextBox,'String','online rule');
set(GLOBAL_OBJ_HANDLES.onlineTextBox, 'Position', ...
    [280,20,160,20],'BackgroundColor','white', 'Fontsize', 14, ...
    'HorizontalAlignment','left');

%OFFLINE
colGreen = 1/255*[0 128 0];
GLOBAL_OBJ_HANDLES.netOffline.fhMaze = subplot(1,3,2);
set(GLOBAL_OBJ_HANDLES.netOffline.fhMaze,'Color','white');
hold on; plotMaze(net.Kx, net.Ky, net.wh); 
hold on; GLOBAL_OBJ_HANDLES.netOffline.PlotLikelihood = scatter(net.X, net.Y,0.1,...
    'c', 'filled', 'markeredgecolor', 'k', 'markerfacecolor','k');
hold on; scatter(  net.initState(1), net.initState(2), 50, 'x', 'linewidth', 3, 'markeredgecolor', 'r');
hold on; scatter( net.targetState(1), net.targetState(2), 50, 'x', 'linewidth', 3, 'markeredgecolor', 'k');
hold on; GLOBAL_OBJ_HANDLES.netOffline.PlotTrajectory = plot([0;0], [0;0], 'linewidth',2, 'color','b');
axis off;
axis([0,net.Kx+1,0,net.Ky+1]);
GLOBAL_OBJ_HANDLES.offlineTextBox = uicontrol('style','text');
set(GLOBAL_OBJ_HANDLES.offlineTextBox,'String','offline rule');
set(GLOBAL_OBJ_HANDLES.offlineTextBox, 'Position', ...
    [680,20,160,20],'BackgroundColor','white', 'Fontsize', 14, ...
    'HorizontalAlignment','left');

%Performance
GLOBAL_OBJ_HANDLES.fhRewards = subplot(1,3,3);
hOptimum = plot([1;1e5], [net.gamma^20;net.gamma^20], 'linewidth',1, 'color','k');
hold on; GLOBAL_OBJ_HANDLES.netOnline.PlotPerf = plot([0;0], [0;0], 'linewidth',2, 'color','b');
hold on; GLOBAL_OBJ_HANDLES.netOffline.PlotPerf = plot([0;0], [0;0], 'linewidth',2, 'color',colGreen);
legend([hOptimum,GLOBAL_OBJ_HANDLES.netOnline.PlotPerf,GLOBAL_OBJ_HANDLES.netOffline.PlotPerf], ...
    'optimum', 'online rule', 'offline rule', 'location', 'southeast');
box off;
ylabel('reward');
xlabel('trials');
axis([1,200,0,0.7]);



%create a plot timer to visualizes the value functions
%*** Plot Event
GLOBAL_OBJ_HANDLES.periodicPlotTimer = timer( ...
    'ExecutionMode', 'fixedRate', ...       % Run timer repeatedly
    'Period', 500e-3, ...                     % in seconds, e.g., 100ms
    'TimerFcn', @PlotTimerTickFcn);             % Specify callback
if strcmp(get(GLOBAL_OBJ_HANDLES.periodicPlotTimer, 'Running'), 'off')
    start(GLOBAL_OBJ_HANDLES.periodicPlotTimer);
end

%create policy update timer for both nets
GLOBAL_OBJ_HANDLES.PolicyUpdateTimer = timer( ...
    'ExecutionMode', 'fixedDelay', ...       % Run timer repeatedly
    'Period', 10e-3, ...                     % in seconds, e.g., 10ms
    'TimerFcn', @PolicyTimerTickFcn);             % Specify callback

% GLOBAL_OBJ_HANDLES.onPolicyUpdateTimer = timer( ...
%     'ExecutionMode', 'fixedRate', ...       % Run timer repeatedly
%     'Period', 80e-3, ...                     % in seconds, e.g., 10ms
%     'TimerFcn', @OnPolicyTimerTickFcn);             % Specify callback
% GLOBAL_OBJ_HANDLES.offPolicyUpdateTimer = timer( ...
%     'ExecutionMode', 'fixedRate', ...       % Run timer repeatedly
%     'Period', 80e-3, ...                     % in seconds, e.g., 10ms
%     'TimerFcn', @OffPolicyTimerTickFcn);             % Specify callback



end


%% Timer Events for On-Policy Updates
function PolicyTimerTickFcn(eventdata, handles)
global GLOBAL_OBJ_HANDLES

OnPolicyTimerTickFcn();
OffPolicyTimerTickFcn();

end

function OnPolicyTimerTickFcn()%eventdata, handles)
global GLOBAL_OBJ_HANDLES

    if GLOBAL_OBJ_HANDLES.flgSimIsRunning < 1
        return;
    end

    net = GLOBAL_OBJ_HANDLES.netOnline; 

    if net.flgRunsFirstTime
        %onpolicy update specific parameters
        net.eta = 0.1;
        Vtrain = -0.1* randn(net.K,1);
        net.beta_est = exp(Vtrain); 
        net.initStateVector = 1/net.K*ones(net.K,1);
        net.stateIdsVector = zeros(1,net.N);
        net.spike_train = zeros(net.K,net.N);
        net.eTrace = zeros(net.K,net.N);
        net.spike_train(net.initStateId,1) = 1;
        net.stateIdsVector(1)  = net.initStateId;%wta_draw(initStateVector);
        %random start
        %stateIdsVector(1)  = wta_draw(initStateVector);
        net.stateTrajectory = zeros(2,net.N);
        net.stateTrajectory(:,1) = [net.X(net.stateIdsVector(1)); net.Y(net.stateIdsVector(1))];
        net.timePriorWeights = repmat(net.gamma.^(0:net.N-1),net.K,1);
        net.firingProb = zeros(net.K,net.N);
        net.firingProb(:,1) = net.initStateVector;
    end
    
    %a new random sample and its updates
    stateIdsVector = net.stateIdsVector;
    spike_train = net.spike_train;
    eTrace = net.eTrace;
    stateTrajectory = net.stateTrajectory;
    
    for n=2:net.N
        
            net.firingProb(:,n) = normalise( ( net.transmat(stateIdsVector(n-1),:)...
                .*net.beta_est').^net.temperature )';
             
            dist = 999; 
            while dist > 1
                stateIdsVector(n) = wta_draw( net.firingProb(:,n) );
                stateTrajectory(:,n) = [net.X(stateIdsVector(n)); net.Y(stateIdsVector(n))];
                dist = (stateTrajectory(1,n) - stateTrajectory(1,n-1))^2 + (stateTrajectory(2,n) - stateTrajectory(2,n-1))^2;
            end
            spike_train( stateIdsVector(n), n) = 1;
            
            eTrace(:,n) = net.gamma*eTrace(:,n-1) + (spike_train(:, n) - net.firingProb(:,n));
            
            %on policy check if obstacle hit or target hit
            [reward maxId flgHitObstacle flgHitTarget flgHitNothing] = net.fnReward(stateTrajectory(:,n));
                 if (flgHitObstacle + flgHitTarget + flgHitNothing) ~= 1
                        error('wrong reward function')
                 end
                    
            if flgHitObstacle
                break;
            end
            if flgHitTarget
               %online update
               deltaV = 1*eTrace(:,n);
               net.Vtrain = net.Vtrain + net.eta*deltaV;   
               net.beta_est = exp(net.Vtrain);%value function or backward message
               break;  
            end
    end   
    maxRewardId = n;
     
    currentReward =  net.gamma^maxRewardId;
    net.rewards = [net.rewards currentReward];
    net.succeeded = [net.succeeded flgHitTarget];
    
    if currentReward > net.maxReward
            net.maxReward = currentReward; 
            net.bestStates = stateTrajectory;
            net.bestMaxRewardId = maxRewardId;
    end
    
    GLOBAL_OBJ_HANDLES.netOnline = net;
end

%% Timer Events for On-Policy Updates
function OffPolicyTimerTickFcn()%eventdata, handles)
global GLOBAL_OBJ_HANDLES

    if GLOBAL_OBJ_HANDLES.flgSimIsRunning < 1
        return;
    end

    net = GLOBAL_OBJ_HANDLES.netOffline; 

    if net.flgRunsFirstTime
        net.flgRunsFirstTime = 0;
        %onpolicy update specific parameters
        net.initStateVector = 1/net.K*ones(net.K,1);
        net.stateIdsVector = zeros(1,net.N);
        net.spike_train = zeros(net.K,net.N);
        net.spike_train(net.initStateId,1) = 1;
        net.stateIdsVector(1)  = net.initStateId;%wta_draw(initStateVector);
        
        net.sumRewardPerState = zeros(net.K,1);
        net.numVisitsPerState = zeros(net.K,1);
    end
    
    %a new random sample and its updates
    [stateIdsVector, stateTrajectory, ...
    rewards, maxRewardId, flgHitObstacle, flgHitTarget, flgHitNothing] = ...
    SampleTrajectory(net.initStateId, net.N, net.X, net.Y, net.transmat, ...
    net.beta_est, net.temperature, net.fnReward);

    if flgHitNothing
        currentReward = 0;
    else 
        currentReward =  net.gamma^maxRewardId;
    end  
    
    net.rewards = [net.rewards nan]; %currentReward
    net.succeeded = [net.succeeded flgHitTarget];
       
    if currentReward > net.maxReward
            net.maxReward = currentReward; 
            net.bestStates = stateTrajectory;
            net.bestMaxRewardId = maxRewardId;
    end
    
    if flgHitTarget %update offpolicy value function
        %r = gamma^(tau - t), where t=0,-1,-2,...,tau-1 for a
        %trajectory of length tau
        r = (net.gamma^maxRewardId)*rewards(maxRewardId)*(net.gamma).^(maxRewardId-1:-1:0);
        [firstVisitIds, timeStepIds] = unique(stateIdsVector(1:maxRewardId));
        net.sumRewardPerState(firstVisitIds) = net.sumRewardPerState(firstVisitIds) + r(timeStepIds)';
        net.numVisitsPerState(firstVisitIds) = 1 + net.numVisitsPerState(firstVisitIds);
            
        %compute unnormalized beta: 
        % 1/M sum r_m, where m=1...M denotes the number of visits of each state/neuron
        net.beta_estOffline = net.sumRewardPerState./max(1,net.numVisitsPerState);
        
        %normalize beta to one
        %net.beta_estOffline =  net.beta_estOffline./max(1,sum(net.beta_estOffline));
    end
    
    GLOBAL_OBJ_HANDLES.netOffline = net;
end

%*** HELPER Functions
function KeyPressFcn(hObject, eventdata, handles)
global GLOBAL_OBJ_HANDLES

    pressedKeyChar = eventdata.Character;

    if pressedKeyChar == 'q' %terminate the simulation
        GLOBAL_OBJ_HANDLES.flgSimIsRunning = 0; 
        if strcmp(get(GLOBAL_OBJ_HANDLES.periodicPlotTimer, 'Running'), 'on')
            stop(GLOBAL_OBJ_HANDLES.periodicPlotTimer);
        end
        if strcmp(get(GLOBAL_OBJ_HANDLES.PolicyUpdateTimer, 'Running'), 'on')
            stop(GLOBAL_OBJ_HANDLES.PolicyUpdateTimer);
        end
%         if strcmp(get(GLOBAL_OBJ_HANDLES.offPolicyUpdateTimer, 'Running'), 'on')
%             stop(GLOBAL_OBJ_HANDLES.offPolicyUpdateTimer);
%         end
        close(GLOBAL_OBJ_HANDLES.hFigKeyStrokes);
        
   elseif pressedKeyChar == 's' 
         disp('Policy Updates started ... (press q to quit)');

         if strcmp(get(GLOBAL_OBJ_HANDLES.PolicyUpdateTimer, 'Running'), 'off')
            start(GLOBAL_OBJ_HANDLES.PolicyUpdateTimer);
         end

         GLOBAL_OBJ_HANDLES.flgSimIsRunning = 1; 

   end
end


%% Timer Events for Plotting Stuff
function PlotTimerTickFcn(eventdata, handles)
global GLOBAL_OBJ_HANDLES

    if GLOBAL_OBJ_HANDLES.flgSimIsRunning < 1
        return;
    end

    
figure(GLOBAL_OBJ_HANDLES.hFigKeyStrokes);
try
    
   dotSizes = compDotSizesFromValueFunction(GLOBAL_OBJ_HANDLES.netOnline.beta_est);
   set(GLOBAL_OBJ_HANDLES.netOnline.PlotLikelihood, 'SizeData', dotSizes);
   
   if ~isempty(GLOBAL_OBJ_HANDLES.netOffline.beta_estOffline)
        dotSizes = compDotSizesFromValueFunction(GLOBAL_OBJ_HANDLES.netOffline.beta_estOffline);
        set(GLOBAL_OBJ_HANDLES.netOffline.PlotLikelihood, 'SizeData', dotSizes);
        
        %compute MAP trajectory
        net = GLOBAL_OBJ_HANDLES.netOffline;
        [stateIdsVector, stateTrajectory, ... 
        rewards, maxRewardId, flgHitObstacle, flgHitTarget, flgHitNothing] = ...
        SampleMAPTrajectory(net.initStateId, net.N, net.X, net.Y, net.transmat, ...
        GLOBAL_OBJ_HANDLES.netOffline.beta_estOffline, net.temperature, net.fnReward);
        mapReward =  net.gamma^maxRewardId;
        
        GLOBAL_OBJ_HANDLES.netOffline.rewards(end) = mapReward;
        
        set(GLOBAL_OBJ_HANDLES.netOffline.PlotTrajectory, 'XData', ...
           stateTrajectory(1,1:maxRewardId));
        set(GLOBAL_OBJ_HANDLES.netOffline.PlotTrajectory, 'YData',...
           stateTrajectory(2,1:maxRewardId));
         
   end
   
   if ~isempty(GLOBAL_OBJ_HANDLES.netOnline.bestStates)
       set(GLOBAL_OBJ_HANDLES.netOnline.PlotTrajectory, 'XData', ...
           GLOBAL_OBJ_HANDLES.netOnline.bestStates(1,1:GLOBAL_OBJ_HANDLES.netOnline.bestMaxRewardId));
       set(GLOBAL_OBJ_HANDLES.netOnline.PlotTrajectory, 'YData',...
           GLOBAL_OBJ_HANDLES.netOnline.bestStates(2,1:GLOBAL_OBJ_HANDLES.netOnline.bestMaxRewardId));
   end
   
   if isempty(GLOBAL_OBJ_HANDLES.netOnline.rewards) && isempty(GLOBAL_OBJ_HANDLES.netOffline.rewards)
       ;
   else
       r = GLOBAL_OBJ_HANDLES.netOnline.rewards;
       rOffline = GLOBAL_OBJ_HANDLES.netOffline.rewards;
       L = max(length(r),length(rOffline));
       
       set(GLOBAL_OBJ_HANDLES.netOnline.PlotPerf, 'XData', 1:length(r));
       set(GLOBAL_OBJ_HANDLES.netOnline.PlotPerf, 'YData', r);
       set(GLOBAL_OBJ_HANDLES.netOffline.PlotPerf, 'XData', 1:length(rOffline));
       set(GLOBAL_OBJ_HANDLES.netOffline.PlotPerf, 'YData', rOffline);
        
       if L > 200 && L < 1000  
           xlim(GLOBAL_OBJ_HANDLES.fhRewards, [1,1000]);drawnow;
       elseif L > 1000 && L < 5000  
           xlim(GLOBAL_OBJ_HANDLES.fhRewards, [1,5000]);drawnow;
       elseif L > 5000 && L < 20000  
           xlim(GLOBAL_OBJ_HANDLES.fhRewards, [1,20000]);drawnow;
       elseif L > 20000
           xlim(GLOBAL_OBJ_HANDLES.fhRewards, [1,200000]);drawnow;
       end
   end

catch err
    err
end

drawnow;

end

function dotSizes = compDotSizesFromValueFunction(beta_est, flgNormalize)

    if nargin < 2
        flgNormalize = 1;
    end
    minDotSize = 0.01;
    maxDotSize = 300;
    if flgNormalize
        dotSizes = max(minDotSize, (maxDotSize/sum(beta_est))*beta_est);
    else
        dotSizes = min(maxDotSize, max(minDotSize, maxDotSize*beta_est));
    end
end

