% The MIT License (MIT)
% Copyright (c) 2016 Elmar Rueckert, Daniel Tanneberg, David Kappel
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE. 

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: rueckert.elmar@gmail.com, tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at
% *
% **********************************************************
function [reward maxId flgHitObstacle flgHitTarget flgHitNothing] = fnRewardMazeToussaint06(states, initState, targetState, obstacleStates)

flgHitObstacle = 0;
flgHitTarget = 0;
flgHitNothing = 0;

N = size(states,2);
maxId = N;
reward = zeros(1,N);


distToTarget = zeros(2,N);
for i=1:2
    distToTarget(i,:) = (targetState(i) - states(i,:)).^2;
end
validIds = find(sum(distToTarget)<1);
 
if ~isempty(validIds)
    N = validIds(1); %check for obstacles only until the target was reached
end


for n=1:N
    hitXIds = find(states(1,n) - obstacleStates(1,:) == 0);
    hitYIds = find(states(2,n) - obstacleStates(2,:) == 0);
    hitIds = intersect(hitXIds, hitYIds);

    if ~isempty(hitIds)
        %keyboard
        flgHitObstacle = 1;
        reward(n) = -1;
        maxId = n;
        return;
    end
end


if  ~isempty(validIds)
    maxId = validIds(1);
    reward(validIds(1)) = 1;
    flgHitTarget = 1;
else
    maxId = N;
    flgHitNothing = 1;
end
    


end
