% The MIT License (MIT)
% Copyright (c) 2016 Elmar Rueckert, Daniel Tanneberg, David Kappel
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE.

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: rueckert.elmar@gmail.com, tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at
% *
% **********************************************************

function [hndArmVisualizer] = plotTwoLinkRobotArm(q, hndArmVisualizer, ...
    flgInputIsInTaskSpace, qTraj_ForDebugging)

if nargin < 1
    q = pi/180*[180,-50]';
end
if nargin < 2
    hndArmVisualizer = initStructure();
else
    figure(hndArmVisualizer.figureHnd);
end

if nargin > 2
    flgUseInvKin = 1;
    
else
    flgUseInvKin = 0;
end

q = validateQ(q);

if flgUseInvKin % use inverse kinematics to compute the joint angles given 
                % the endeffector position(s) 
    x = q;
%     keyboard
    %sample initial pose
    if 1==0
    numSamples = 10000;
    qSamples = 2*pi*rand(2,numSamples);
    efPoses = computeEndEffectorPositions(qSamples, ...
        hndArmVisualizer.linkLengths, hndArmVisualizer.PARAM_ORIGIN_JOINT_ONE);
    
    xSamples = efPoses(3).EF;
    xTrue = [x(1,1)*ones(1,numSamples); x(2,1)*ones(1,numSamples)];
    distances = sum((xSamples-xTrue));
    [minDist, bestSampleId] = min(distances);
    if minDist > 1 %1 deg
        warning('Warning: the estimation of the initial pose using sampling and fwd kinematics was poor!');
    end
    initialPose = qSamples(:, bestSampleId);
    initialPose*180/pi
    else
        initialPose = pi/180*[180, -140]';
    end
    %should be q0 = pi/180*[180, -140]';
    
    if nargin < 4
        q = computeJointsFromEndEffectors(initialPose, x, hndArmVisualizer);
    else
        q = computeJointsFromEndEffectors(initialPose, x, hndArmVisualizer, qTraj_ForDebugging);
    end
    
    %remove the first 20ms which are corrupted because of the Jacobian
    %inverse
    q = q(:,21:end);
    
end


resetEndEffectorTrace(hndArmVisualizer);

hndArmVisualizer = plotArm(q, hndArmVisualizer);

%store the trace
xData = get(hndArmVisualizer.jointElements.efTrace.lineElement, 'XData');
yData = get(hndArmVisualizer.jointElements.efTrace.lineElement, 'YData');

hndArmVisualizer.efTrace = [xData; yData];

end

function resetEndEffectorTrace(hndArmVisualizer)
    %rest EndEffectorTrace
    efTrace = hndArmVisualizer.jointElements.efTrace;
    set(efTrace.lineElement, 'XData', [], 'YData', []);
end

function hndArmVisualizer = plotArm(q, hndArmVisualizer)
    [dimT] = size(q,2);
    efs = computeEndEffectorPositions(q, hndArmVisualizer.linkLengths, hndArmVisualizer.PARAM_ORIGIN_JOINT_ONE);
    if dimT <= 1
         hndArmVisualizer = plotArmStep(q, efs, hndArmVisualizer);
    else
        for t=1:dimT
             efsAtTimeT = getSingleTimeStepEFPose(efs, t);
             hndArmVisualizer = plotArmStep(q(:,t), efsAtTimeT, hndArmVisualizer);
             pause(hndArmVisualizer.PARAM_TRAJ_PAUSE_IN_MS/1000);
        end
    end
end

function hndArmVisualizer = plotArmStep(q, efs, hndArmVisualizer)

    %1. links
    
%     linkElements.linkPatches = linkPatches;
    linkPatches = hndArmVisualizer.linkElements.linkPatches;
    oldRotationAngles = hndArmVisualizer.linkElements.lastRotationAngles;
    newRotationAngles = [0;cumsum(q)];
    oldOrigins = hndArmVisualizer.linkElements.lastOrigins;
    newOrigins = [hndArmVisualizer.PARAM_BASE_POSE, efs(1).EF, efs(2).EF, efs(3).EF];
    for i = 1:size(linkPatches,2)
        rotate(linkPatches(i).patchElement,[0,0,1],oldRotationAngles(i)*180/pi, [oldOrigins(:,i);0]');
        if i == 3 %for speed up only, otherwise no if is required
            oldX = get(linkPatches(i).patchElement,'XData');
            oldY = get(linkPatches(i).patchElement,'YData');
            set(linkPatches(i).patchElement, 'XData', oldX - oldOrigins(1,i)+newOrigins(1,i), ...
            'YData',  oldY - oldOrigins(2,i)+newOrigins(2,i));
        end
        rotate(linkPatches(i).patchElement,[0,0,1],-newRotationAngles(i)*180/pi, [newOrigins(:,i);0]');
    end
    hndArmVisualizer.linkElements.lastRotationAngles = newRotationAngles;
    hndArmVisualizer.linkElements.lastOrigins = newOrigins;
    
%     linkElements.linkLines = linkLines;
    linkLines = hndArmVisualizer.linkElements.linkLines;
    efLast = hndArmVisualizer.PARAM_BASE_POSE;
    for i=1:3
        ef = efs(i).EF;
        set(linkLines(i).lineElement, 'XData', [efLast(1), ef(1)], 'YData', [efLast(2), ef(2)]);
        efLast = ef;
    end
    
    
    %2. joints
    
%     jointElements.largeCircles = largeCircles;
    largeCircles = hndArmVisualizer.jointElements.largeCircles;
    lastEFs = hndArmVisualizer.jointElements.lastEFs;
    for i=1:3
        oldX = get(largeCircles(i).patchElement,'XData');
        oldY = get(largeCircles(i).patchElement,'YData');
        
        set(largeCircles(i).patchElement, 'XData', oldX - lastEFs(i).EF(1) + efs(i).EF(1), ...
            'YData',  oldY - lastEFs(i).EF(2) + efs(i).EF(2) );
    end
    hndArmVisualizer.jointElements.lastEFs = efs;
    
%     jointElements.smallCircles = smallCircles;
    smallCircles = hndArmVisualizer.jointElements.smallCircles;
    for i=1:3
        set(smallCircles(i).scatterElement, 'XData', efs(i).EF(1), 'YData', efs(i).EF(2) );
    end
    
%     jointElements.efTrace = efTrace;
    efTrace = hndArmVisualizer.jointElements.efTrace;
    oldX = get(efTrace.lineElement,'XData');
    oldY = get(efTrace.lineElement,'YData');
    set(efTrace.lineElement, 'XData', [oldX, efs(3).EF(1)], 'YData', [oldY, efs(3).EF(2)]);
end


% *** CHANGE PLOTTING PARAMETERS HERE ***
function hndArmVisualizer = initStructure()

    %PARAMETERS of the VISUALIZATION
    PARAM_LABEL_FONT_SIZE = 20;
    PARAM_SMALL_CIRCLE_SIZE_IN_UNITS = 10*[1; 1; 1];        %in meter
    PARAM_LARGE_CIRCLE_SIZE = 0.5*[0.2; 0.1; 0.05];         %in meter
    PARAM_LARGE_CIRCLE_COLOR =  [0.5 0.5 0.5];              %in meter
    PARAM_ORIGIN_JOINT_ONE = [-1; 0.5];
    hndArmVisualizer.PARAM_ORIGIN_JOINT_ONE = PARAM_ORIGIN_JOINT_ONE;
    PARAM_BASE_POSE = PARAM_ORIGIN_JOINT_ONE + [0; 0.25];   %base height in meter
    hndArmVisualizer.PARAM_BASE_POSE = PARAM_BASE_POSE;
    PARAM_BASE_WIDTH = [0.4; PARAM_LARGE_CIRCLE_SIZE(1)];   %bottom, top in meter
    PARAM_LINE_WIDTH = 1.5;
    PARAM_PATCH_COLOR = 1/255*[0 102 204];
    PARAM_TRACE_COLOR = 1/255*[205,92,92];
    hndArmVisualizer.PARAM_TRAJ_PAUSE_IN_MS = 3;
    PARAM_EF_TRACE_LINE_WIDTH = 3;

    hndArmVisualizer.axis = [-1.5, 1.5, -1.2, 0.8];
    h = 480;
    w = h*(hndArmVisualizer.axis(2) - hndArmVisualizer.axis(1))/(hndArmVisualizer.axis(4) - hndArmVisualizer.axis(3));

    hndArmVisualizer.figureHnd = figure();
    set(hndArmVisualizer.figureHnd, 'position',[640 558 w h]);
    hndArmVisualizer.linkLengths = [1.2; 1];                %link lengths in meter
    
    axis(hndArmVisualizer.axis);
    hndArmVisualizer.gca = gca;
        
    %graphical elements
    q = pi/180*[180,0]';
    efs = computeEndEffectorPositions(q, hndArmVisualizer.linkLengths, hndArmVisualizer.PARAM_ORIGIN_JOINT_ONE);
   
    hndArmVisualizer.linkElements = createLinkElements(q, efs, ...
        PARAM_BASE_POSE, PARAM_BASE_WIDTH, ...
        PARAM_LINE_WIDTH, PARAM_PATCH_COLOR, ...
        PARAM_LARGE_CIRCLE_SIZE, hndArmVisualizer.linkLengths, ...
        PARAM_ORIGIN_JOINT_ONE);
    
    hndArmVisualizer.jointElements = createJointElements(q, efs, ...
        PARAM_SMALL_CIRCLE_SIZE_IN_UNITS, PARAM_LARGE_CIRCLE_SIZE, ...
        PARAM_LARGE_CIRCLE_COLOR, PARAM_EF_TRACE_LINE_WIDTH, ...
        PARAM_TRACE_COLOR, PARAM_ORIGIN_JOINT_ONE);
    
    %format
%     grid on;
    %grid minor;
    set(hndArmVisualizer.figureHnd, 'color','white');
    set(hndArmVisualizer.figureHnd,'PaperPositionMode','auto');
    xlabel('x','fontsize', PARAM_LABEL_FONT_SIZE);
    ylabel('y','fontsize', PARAM_LABEL_FONT_SIZE);
    set(hndArmVisualizer.gca,'fontsize', PARAM_LABEL_FONT_SIZE);
    set(hndArmVisualizer.gca,'XMinorTick','on','YMinorTick','on');
    box on;
    set(gca,'YDir','reverse');    
end


% *** HELPER FUNCTIONS ***
function q = validateQ(q)
    if size(q,1) == 1
        q = q';
    end

    [dimQ] = size(q,1);

    if dimQ ~= 2 && dimQ ~= 4 
        error('invalid argument [q]: input joint angles state or trajectory must have 2 or 4 rows!');
    end

    if dimQ == 4
        q = q(1,3,:);
    end
end

function linkElements = createLinkElements(q, efs, ...
    PARAM_BASE_POSE, PARAM_BASE_WIDTH, ...
    PARAM_LINE_WIDTH, PARAM_PATCH_COLOR, PARAM_LARGE_CIRCLE_SIZE,  ...
    linkLengths, PARAM_ORIGIN_JOINT_ONE)
    
    linkPatches = struct([]);
    patchWidths = [PARAM_BASE_WIDTH(1); 2*PARAM_LARGE_CIRCLE_SIZE];
    patchLengths = [(PARAM_ORIGIN_JOINT_ONE(2)-PARAM_BASE_POSE(2)); linkLengths];
    %keyboard
    origins = [PARAM_BASE_POSE, efs(1).EF, efs(2).EF, efs(3).EF];
%     rotationAngles = [0;-pi/2+cumsum(q)];
    rotationAngles = [0;cumsum(q)];

    
    for i=1:3%3:-1:1
        o = origins(:,i);
        pw_bottom = patchWidths(i);
        pw_top = patchWidths(i+1);
        pl = patchLengths(i);
        
        x = [o(1)-pw_bottom/2; o(1)+pw_bottom/2; o(1)+pw_top/2; o(1)-pw_top/2];
        y = [o(2); o(2); o(2)+pl;o(2)+pl];
        hold on;
        linkPatches(i).patchElement = patch(x, y, PARAM_PATCH_COLOR); 
        hold off;
        %ROTATE PATH
        rotate(linkPatches(i).patchElement,[0,0,1],-rotationAngles(i)*180/pi, [o;0]');
    end

    linkLines = struct([]);
    efLast = PARAM_BASE_POSE;
    
    for i=1:3
        ef = efs(i).EF;
        hold on;
        linkLines(i).lineElement = line([efLast(1), ef(1)], [efLast(2), ef(2)]);
        set(linkLines(i).lineElement, 'color', 'k', 'linestyle', '--', 'linewidth', PARAM_LINE_WIDTH);
        hold off;
        efLast = ef;
    end
    
    linkElements.lastOrigins = origins;
    linkElements.lastRotationAngles = rotationAngles;
    linkElements.linkPatches = linkPatches;
    linkElements.linkLines = linkLines;
end

function jointElements = createJointElements(q, efs, ...
    PARAM_SMALL_CIRCLE_SIZE_IN_UNITS, PARAM_LARGE_CIRCLE_SIZE, ...
    PARAM_LARGE_CIRCLE_COLOR, PARAM_EF_TRACE_LINE_WIDTH, ...
    PARAM_TRACE_COLOR, PARAM_ORIGIN_JOINT_ONE)

    largeCircles = struct([]);
    for i=1:3
        ef = efs(i).EF;
        [x, y] = createCirclePolygon(ef, PARAM_LARGE_CIRCLE_SIZE(i));
        
        hold on;
        largeCircles(i).patchElement = patch(x, y, PARAM_LARGE_CIRCLE_COLOR); 
        hold off;
    end
    
    smallCircles = struct([]);
    for i=1:3
        ef = efs(i).EF;
        hold on;
        smallCircles(i).scatterElement = scatter(ef(1), ef(2), ...
            PARAM_SMALL_CIRCLE_SIZE_IN_UNITS(i), 'c', 'filled', ...
            'markeredgecolor', 'k', 'markerfacecolor', 'k');
        hold off;
    end

    %EndEffector trace
    ef = efs(3).EF;
    hold on;
    efTrace.lineElement = line([ef(1), ef(1)], [ef(2), ef(2)]);
    set(efTrace.lineElement, 'color', PARAM_TRACE_COLOR, 'linewidth', PARAM_EF_TRACE_LINE_WIDTH);
    hold off;
    
    jointElements.lastEFs = efs;
    jointElements.largeCircles = largeCircles;
    jointElements.smallCircles = smallCircles;
    jointElements.efTrace = efTrace;
    
end

% FORWARD KINEMATICS
function efPoses = computeEndEffectorPositions(q, linkLengths, PARAM_ORIGIN_JOINT_ONE)
    dimT = size(q,2);
    l1 = linkLengths(1);
    l2 = linkLengths(2);
    efPoses = struct([]);
    efPoses(1).EF = [PARAM_ORIGIN_JOINT_ONE(1)*ones(1,dimT); PARAM_ORIGIN_JOINT_ONE(2)*ones(1,dimT)];
    %counter clockwise system
%     efPoses(2).EF = [l1*cos(q(1,:)); l1*sin(q(1,:))];
%     efPoses(3).EF = [l1*cos(q(1,:))+l2*cos(q(1,:) + q(2,:)); l1*sin(q(1,:))+l2*sin(q(1,:) + q(2,:))];
    %clockwise system
    efPoses(2).EF = efPoses(1).EF + [l1*sin(q(1,:)); l1*cos(q(1,:))];
    efPoses(3).EF = efPoses(2).EF + [l2*sin(q(1,:) + q(2,:)); l2*cos(q(1,:) + q(2,:))];
    %efPoses(3).EF = [y] = getForwardKinematics(q, linkLengths, 2);
end

% INVERSE KINEMATICS
function q = computeJointsFromEndEffectors(lastQ, x, hndArmVisualizer, qTraj_ForDebugging)
    [dimX, dimT] = size(x);
    q = [lastQ(1)*ones(1,dimT); lastQ(2)*ones(1,dimT)];
    lambda = 1e-3;
    
    for t=1:dimT-1

        if nargin < 4
            currentJointState = q(:,t);
        else
            currentJointState = qTraj_ForDebugging(:,t);
        end

        [J, ef]  = getJacobian(currentJointState', hndArmVisualizer.linkLengths, ...
            length(hndArmVisualizer.linkLengths), hndArmVisualizer.PARAM_ORIGIN_JOINT_ONE);
        ef = ef'; 
        e = x(:,t+1)-ef;

        deltaQ =  J'*(J* J' + lambda*eye(dimX))^(-1)*e;
        q(:,t+1) = minus_pi_to_pi(currentJointState+deltaQ);
    end
end

function x = minus_pi_to_pi(xin)
%% Transformation function

    x_phi = xin;

    i=find(x_phi>pi); 
    x_phi(i)=x_phi(i)-2*pi; 

    i=find(x_phi<-pi); 
    x_phi(i)=x_phi(i)+2*pi; 

    x = x_phi;
    
end


function efPose = getSingleTimeStepEFPose(efPosesTrajectory, timeIndex)
    efPose = struct([]);
    efPose(1).EF = efPosesTrajectory(1).EF(:,timeIndex);
    efPose(2).EF = efPosesTrajectory(2).EF(:,timeIndex);
    efPose(3).EF = efPosesTrajectory(3).EF(:,timeIndex);
end


function [x,y] = createCirclePolygon(center, r)
    N = round(r*1000); % Number of side
    theta = linspace(0,2*pi,N)';
    x = cos(theta)*r(1,:) + ones(N,1)*center(1);
    y = sin(theta)*r(end,:) + ones(N,1)*center(2);
end

%GENERIC N-LINK ARM FORWARD KINEMATICS and JACOBIAN
%Thanks to Paraschos Alexander, paraschos@ias.tu-darmstadt.de
function [y] = getForwardKinematics(theta, lengths, numLink, PARAM_ORIGIN_JOINT_ONE)

    len = lengths;
    y = PARAM_ORIGIN_JOINT_ONE';%zeros(size(theta, 1),2);
    for i = 1:numLink
       y = y + [sin(sum(theta(:,1:i), 2)), cos(sum(theta(:, 1:i), 2))] * len(i);
    end

end
function [J, si]  = getJacobian(theta, lengths, numLink, PARAM_ORIGIN_JOINT_ONE) 
% Get the jacobian at a given angle position q for a given link numLink

    si = getForwardKinematics(theta, lengths, numLink, PARAM_ORIGIN_JOINT_ONE);
    J  = zeros(2,numLink);

    for j = 0:(numLink - 1)
        pj = PARAM_ORIGIN_JOINT_ONE';%[0, 0];
        for i = 1:j
           pj = pj + [sin(sum(theta(1:i))), cos(sum(theta(1:i)))] * lengths(i);
        end
        pj = -(si - pj);
        J([1 , 2], j + 1) = [-pj(2) pj(1)];
    end

end

