% The MIT License (MIT)
% Copyright (c) 2016 Elmar Rueckert, Daniel Tanneberg, David Kappel
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE.

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: rueckert.elmar@gmail.com, tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at
% *
% **********************************************************
function net = GenerateSNNDiscreteTaskJoint(task)

    net.W = task.cartW;
    net.b = task.cartb;
    
    net.refractoryPeriodeInSeconds = 10e-3; %10ms
    net.simTimeStep = 1e-3;
    net.beta = 1; %inverse temperature
    net.numStateNeurons = task.numStateNeurons;

    net.transferFunction = @(x) x;

    net.simulate = @(numberOfTimeSteps_)localSimulate(numberOfTimeSteps_, net);

end

function spikeTrains = localSimulate(numberOfTimeSteps, net)

    numNeurons = size(net.W,1);
    cartSpikeTrain = sparse(numNeurons, numberOfTimeSteps);
    
    tau = net.refractoryPeriodeInSeconds/net.simTimeStep;
    refractoryState = zeros(numNeurons,1);
    for l=1:numberOfTimeSteps
        active = find(refractoryState > 0);
        inactive = find(refractoryState <= 0);
        
        %advance internal state of neurons that are in their ref. periode
        refractoryState(active) = max(zeros(length(active),1),-1+refractoryState(active));
        currSpikes = refractoryState > 0;
        
        %update membran potential of all inactive neurons
        if l==1
            u = net.b(inactive,l);
        else
            u = net.W(:,inactive)'*currSpikes;           
            u = u + net.b(inactive,l);
        end
        
        uSample = u;
        ableToSpikeIds = find(uSample ~= 0);
        
        % generate spikes
        cartSpikes = rand(length(ableToSpikeIds),1) < net.transferFunction((net.beta)*(uSample(ableToSpikeIds)));
       
        refractoryState(inactive(ableToSpikeIds),1) = tau*cartSpikes;
        cartSpikeTrain(inactive(ableToSpikeIds),l) = cartSpikes;   
        
        currSpikes = refractoryState > 0;

    end
    spikeTrains = cartSpikeTrain; 
end
