% The MIT License (MIT)
% Copyright (c) 2016  Daniel Tanneberg, David Kappel, Elmar Rueckert
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE. 

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at, rueckert.elmar@gmail.com
% *
% **********************************************************
function filteredSpikeTrain = SNNGaussianFilter(spikeTrain)

[numNeurons, numTimeSteps] = size(spikeTrain);

windowSize = 100; % 100
gaussFilter = gausswin(windowSize); %gaussian filter

gaussFilter = gaussFilter ./ sum(gaussFilter);
filteredSpikeTrain = zeros(numNeurons, numTimeSteps);
for i=1:numNeurons    
    tmp = sparseconv(spikeTrain(i,:)', gaussFilter)';
    filteredSpikeTrain(i,:) = tmp(floor(windowSize/2):floor(windowSize/2)+numTimeSteps-1);

end 


end


