% The MIT License (MIT)
% Copyright (c) 2016  Daniel Tanneberg, David Kappel, Elmar Rueckert
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE. 

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: tanneberg@ias.tu-darmstadt.de, david.kappel@igi.tugraz.at, rueckert.elmar@gmail.com
% *
% **********************************************************
function [t_start, t_end] = cutTraj_new(plan, task, time_window)
% find start & end point in the given trajectory

    x = plan(1,:);
    y = plan(2,:);
    
    target = task.targetState;
    init = task.initialState;
    
    % init
    dists = pdist2(init',[x;y]');   
    
    gaussFilter = gausswin(time_window); %gaussian filter   
    gaussFilter = gaussFilter ./ sum(gaussFilter);
    tmp = conv(dists', gaussFilter, 'same')';
   
    [min_dist,t_start] = min(tmp);
   
    % target
    dists = pdist2(target',[x;y]');   
    
    gaussFilter = gausswin(time_window); %gaussian filter   
    gaussFilter = gaussFilter ./ sum(gaussFilter);
    tmp = conv(dists', gaussFilter, 'same')';

    [min_dist,t_end] = min(tmp);
end

