% The MIT License (MIT)
% Copyright (c) 2016 David Kappel, Elmar Rueckert, Daniel Tanneberg
% 
% Permission is hereby granted, free of charge, to any person obtaining a 
% copy of this software and associated documentation files (the "Software"), 
% to deal in the Software without restriction, including without limitation 
% the rights to use, copy, modify, merge, publish, distribute, sublicense, 
% and/or sell copies of the Software, and to permit persons to whom the 
% Software is furnished to do so, subject to the following conditions:
% 
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
% 
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
% DEALINGS IN THE SOFTWARE.

% **********************************************************
% *
% * Supplementary Code to 
% *                       Recurrent Spiking Networks Solve Planning Tasks, 
% *                       Scientific Reports, 6, 21142, Nature Publishing Group.
% * 
% *                       Rueckert, Elmar and Kappel, David and Tanneberg, Daniel
% *                       and Pecevski, Dejan and Peters, Jan (2016).
% * 
% * If you use this code, please cite the paper!
% * 
% * contact: david.kappel@igi.tugraz.at, rueckert.elmar@gmail.com, tanneberg@ias.tu-darmstadt.de
% *
% **********************************************************

function k = wta_draw( P, numRuns )
%
% k = wta_draw( P )
%
% Draw a random from a distribution P.
% P is assumed to be normalised over columns, so for
% each column a sample is drawn. Output has the same
% dimensionality as P.
%
% 05.08.2011
%
if nargin < 2
    k = find( cumsum(P) > rand(), 1 );
    return;
end

k = zeros(numRuns,1);
for i=1:numRuns
    k(i) = find( cumsum(P) > rand(), 1 );
end
counts = hist(k, 1:size(P,1));
    
[val k] = max(counts);